--[[

BuffProtectorGnome, adds a confirmation dialog before you cancel selected buffs.
Copyright (C) 2007 rabbit.magtheridon@gmail.com

This program is free software; you can redistribute it and/or
modify it under the terms of the GNU General Public License
as published by the Free Software Foundation; either version 2
of the License, or (at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.

]]

local L = AceLibrary("AceLocale-2.2"):new("BuffProtectorGnome")
local gratuity = AceLibrary("Gratuity-2.0")

local NUM_QUIPS = 10

L:RegisterTranslations("enUS", function() return {
	-- Protected buffs
	["Spirit of Zandalar"] = true,
	["Warchief's Blessing"] = true,
	["Rallying Cry of the Dragonslayer"] = true,
	["Fengus' Ferocity"] = true,
	["Mol'dar's Moxie"] = true,
	["Slip'kik's Savvy"] = true,
	-- Flasks
	["Supreme Power"] = true,
	["Distilled Wisdom"] = true,
	["Chromatic Resistance"] = true,
	["Flask of the Titans"] = true,
	-- Pots
	["Arcane Elixir"] = true,
	["Elixir of the Mongoose"] = true,
	["Shadow Power"] = true,
	["Greater Arcane Elixir"] = true,
	["Mana Regeneration"] = true,
	["Regeneration"] = true,
	["Sheen of Zanza"] = true,
	["Spirit of Zanza"] = true,
	["Swiftness of Zanza"] = true,
	["Nature Protection"] = true,
	["Shadow Protection"] = true,
	["Fire Protection"] = true,
	["Holy Protection"] = true,
	["Arcane Protection"] = true,
	["Frost Protection"] = true,

	-- Addon translation
	["list"] = true,
	["Lists the protected buffs."] = true,
	["add"] = true,
	["Protects the given buff."] = true,
	["<buff name>"] = true,
	["remove"] = true,
	["Removes the given buff from the protected list."] = true,
	["all"] = true,
	["Toggle whether all buffs except those without duration should be protected."] = true,

	["Cancel buff"] = true,
	["Protect"] = true,

	["Your BuffProtectorGnome is currently protecting these buffs:"] = true,
	["%s is now protected by the BuffProtectorGnome."] = true,
	["%s is no longer protected."] = true,

	[1] = "As the battle roars in the distant, your BuffProtectorGnome wonders if you really want to remove your %s?",
	[2] = "Out for an evening stroll, the BuffProtectorGnome is not here. But he has taught you how to handle %s.",
	[3] = "Never giving you a moments peace, here's your BuffProtectorGnome pestering you about %s again.",
	[4] = "Like you don't know the answer already, your BuffProtectorGnome is asking you about %s again.",
	[5] = "%s is starting to annoy you, and so is your BuffProtectorGnome.",
	[6] = "With the battle roars from your fellow mates in the distant, you wonder if you'll need %s.",
	[7] = "Never underestimate your BuffProtectorGnome. Or your %s.",
	[8] = "%s has saved you in the past. Your BuffProtectorGnome seems reluctant to let it go.",
	[9] = "Tuning down his iPod, you can still hear the lyrics to Eminems 'Run Rabbit Run', your BuffProtectorGnome asks if you really want to remove %s?",
	[10] = "The BuffProtectorGnome looks at you, he's not amused. Maybe you should just remove %s from your list of protected buffs sooner rather than later.",
} end)

BuffProtectorGnome = AceLibrary("AceAddon-2.0"):new("AceDB-2.0", "AceConsole-2.0", "AceHook-2.1")

local options = {
	type = "group",
	args = {
		list = {
			name = L["list"], type = "execute",
			desc = L["Lists the protected buffs."],
			func = function() BuffProtectorGnome:ListBuffs() end,
		},
		add = {
			name  = L["add"], type = "text",
			desc  = L["Protects the given buff."],
			usage = L["<buff name>"],
			set   = function(text) BuffProtectorGnome:Protect(text) end,
			get   = false,
		},
		remove = {
			name  = L["remove"], type = "text",
			desc  = L["Removes the given buff from the protected list."],
			usage = L["<buff name>"],
			set   = function(text) BuffProtectorGnome:Ignore(text) end,
			get   = false,
		},
		all = {
			name = L["all"], type = "toggle",
			desc = L["Toggle whether all buffs except those without duration should be protected."],
			set = function(v) BuffProtectorGnome.db.profile.protectAll = v end,
			get = function() return BuffProtectorGnome.db.profile.protectAll end,
		},
	},
}

function BuffProtectorGnome:OnInitialize()
	local revision = tonumber((string.gsub("$Revision: 27270 $", "^.-(%d+).-$", "%1"))) or 1
	if not self.version then self.version = "1.0" end
	self.version = self.version .. "." .. revision
	self.revision = revision

	self:RegisterDB("BuffProtectorGnomeDB")
	self:RegisterDefaults("profile", {
		protectAll = nil,
		protect = {
			-- Buffs
			[L["Spirit of Zandalar"]] = true,
			[L["Warchief's Blessing"]] = true,
			[L["Rallying Cry of the Dragonslayer"]] = true,
			[L["Fengus' Ferocity"]] = true,
			[L["Mol'dar's Moxie"]] = true,
			[L["Slip'kik's Savvy"]] = true,
			-- Flasks
			[L["Supreme Power"]] = true,
			[L["Distilled Wisdom"]] = true,
			[L["Chromatic Resistance"]] = true,
			[L["Flask of the Titans"]] = true,
			-- Pots
			[L["Arcane Elixir"]] = true,
			[L["Elixir of the Mongoose"]] = true,
			[L["Shadow Power"]] = true,
			[L["Greater Arcane Elixir"]] = true,
			[L["Mana Regeneration"]] = true,
			[L["Regeneration"]] = true,
			[L["Sheen of Zanza"]] = true,
			[L["Spirit of Zanza"]] = true,
			[L["Swiftness of Zanza"]] = true,
			[L["Nature Protection"]] = true,
			[L["Shadow Protection"]] = true,
			[L["Fire Protection"]] = true,
			[L["Holy Protection"]] = true,
			[L["Arcane Protection"]] = true,
			[L["Frost Protection"]] = true,
		},
	})

	self:RegisterChatCommand({"/buffprotectorgnome", "/bpg"}, options, "BUFFPROTECTORGNOME")
end

function BuffProtectorGnome:OnEnable()
	-- Yes, we taint, but according to WoWWiki, tainting this API call is safe.
	self:Hook("CancelPlayerBuff", true)

	if not StaticPopupDialogs then
		StaticPopupDialogs = {}
	end

	if not StaticPopupDialogs["BuffProtectorGnomeQuery"] then
		StaticPopupDialogs["BuffProtectorGnomeQuery"] = {
			button1 = L["Cancel buff"],
			button2 = L["Protect"],
			sound = "AuctionWindowClose",
			whileDead = 1,
			hideOnEscape = 1,
			timeout = 30,
			showAlert = 1,
			OnAccept = function(data)
				self.hooks["CancelPlayerBuff"](data)
			end,
			OnCancel = function(data, reason)
				if reason == "timeout" then
					self.hooks["CancelPlayerBuff"](data)
				end
			end
		}
	end

end

function BuffProtectorGnome:CancelPlayerBuff(buffInput, ...)
	if type(buffInput) ~= "string" and type(buffInput) ~= "number" then
		self.hooks["CancelPlayerBuff"](buffInput, ...)
		return
	end

	-- aura = 1 means we default to not protecting if called from /cancelaura.
	local aura = 1
	local buffName = nil
	-- If it's a string, we're called from /cancelaura or some script.
	if type(buffInput) == "string" then
		buffName = buffInput
	else -- Should be a number, in which case it should be the index.
		local tmp
		tmp, aura = GetPlayerBuff(tonumber(buffInput))
		gratuity:SetPlayerBuff(tmp, aura)
		buffName = gratuity:GetLine(1)
	end

	if (self.db.profile.protectAll and aura ~= 1) or self:IsProtected(buffName) then
		self:QueryPlayer(buffName, buffInput)
	else
		self.hooks["CancelPlayerBuff"](buffInput, ...)
	end
end

function BuffProtectorGnome:ListBuffs()
	self:Print(L["Your BuffProtectorGnome is currently protecting these buffs:"])
	for k, v in pairs(self.db.profile.protect) do
		self:Print(k)
	end
end

function BuffProtectorGnome:Protect(buff)
	self.db.profile.protect[buff] = true
	self:Print(string.format(L["%s is now protected by the BuffProtectorGnome."], buff))
end

function BuffProtectorGnome:Ignore(buff)
	self.db.profile.protect[buff] = nil
	self:Print(string.format(L["%s is no longer protected."], buff))
end

function BuffProtectorGnome:IsProtected(buff)
	if type(buff) ~= "string" or buff == "" then return false end
	return self.db.profile.protect[buff] and true or false
end

function BuffProtectorGnome:GetQuip(input)
	local rand = 1
	repeat rand = math.random(1, NUM_QUIPS) until L[rand] ~= nil
	return string.format(L[rand], "|cffd9d919"..input.."|r")
end

function BuffProtectorGnome:QueryPlayer(buff, index)
	StaticPopupDialogs["BuffProtectorGnomeQuery"].text = self:GetQuip(buff)
	local popup = StaticPopup_Show("BuffProtectorGnomeQuery")
	if popup then popup.data = index end
end

